/*************************************************************************
 * Copyright(c) 2013~2021  El Condor
 * Asyncronous WEB with Server SPIFFS Filesystem 
 * Version 0.1.0 27 April 2021
 ************************************************************************/
#include <WiFi.h>
#include <AsyncTCP.h>
#include "ESPAsyncWebServer.h"
// temperature, humidity sensor *****************************************
#include "DHT.h"
#define DHTPIN 4        // digital pin that connect DHT22
#define DHTTYPE DHT11   // there are multiple kinds of DHT sensors
DHT dht(DHTPIN, DHTTYPE);
#define LENGTHWORKBUFFER 100
#define day 86400
char workBuffer [LENGTHWORKBUFFER + 1];
uint32_t clck;
// SSID & Password ******************************************************
const char* ssid = "Sermig_Condor";
const char* password = "";
// IP Address ***********************************************************
IPAddress local_ip(192, 168, 1, 1);
IPAddress gateway(192, 168, 1, 1);
IPAddress subnet(255, 255, 255, 0);
AsyncWebServer server(80);  // Object of WebServer(HTTP port, 80 is default)
#include "fileSystem.h"
unsigned long ag_Clock = 0;
void setTime(long t) {  // sets the time of day
  ag_Clock = ag_Clock - (ag_Clock % day) + t - millis()/1000; // new time
}
char * getTime() {
  unsigned long secs = ag_Clock+millis()/1000;
  static char timeOfDay[12] = "";
  unsigned long h = (secs % day) / 3600;
  unsigned long m = ((secs % day) - h*3600)/60;
  unsigned long s = secs%60;
  sprintf(timeOfDay,"%02lu:%02lu:%02lu",h,m,s);
  return timeOfDay;
}
int32_t toSeconds(const char * time) {    // convert hh[:mm[:ss]] to a number of seconds
  int hh=-1,mm=0,ss=0;
  sscanf(time,"%u:%u:%u",&hh,&mm,&ss);
  if (hh>23||mm>59||ss>59) return -1;
  return (int32_t) hh*3600+mm*60+ss;
}
char * getTemperature() {
    float h = dht.readHumidity();
    float t = dht.readTemperature();         
    static char answer[50] = "";
    sprintf(answer,"Humidity: %.1f %%. Temperature: %-.1f C",h,t);
    return answer;
}
void serialListener( void * pvParameters ) {  // this runs in core 0
  Serial.print("Serial Listener running on core ");
  Serial.println(xPortGetCoreID());
  String helpString = "Commands:\n d = files list\n r = reset board\n t = show humidity and temperature";
  Serial.println(helpString);
  delay(100);
  for(;;){
    while (Serial.available() > 0) {
      char cmd = Serial.read() | 0b00100000;  // decapitalize
      switch (cmd) {
        case 100:         // d
          dir_ls();
          break;
        case 114:         // r reset Board
          Serial.print("\nReset board\n");
          delay(200);
          ESP.restart();
          break;
        case 116:         // t get temperature/humidity
          sprintf(workBuffer, "%s %s", getTime(),getTemperature());
          Serial.println(workBuffer);
          break;
        default:
          Serial.println(helpString);
      }
    }
    vTaskDelay(150 / portTICK_PERIOD_MS);   // 150 millisecond
  }
}
void setup() {
  Serial.begin(9600);
  dht.begin();  // DHT22 sensor
  if(!SPIFFS.begin(true)) {
    Serial.println("An Error has occurred while mounting SPIFFS");
    return;
  }
  WiFi.softAP(ssid, password);
  delay(2000);  // to avoid crash on WiFi connession
  WiFi.softAPConfig(local_ip, gateway, subnet);
  Serial.printf("Connect to Access Point: %s\n",ssid);
  server.on("/", HTTP_ANY, [](AsyncWebServerRequest *request) {
    int args = request->args();
   if(request->hasArg("fnz")) {
     String type = request->arg("fnz");   
     AsyncResponseStream *response = request->beginResponseStream("text/html");   
     if (type == "Dir") {
        response->print(dir());
        request->send(response);
     } else if (type == "Clock") {       
        setTime(toSeconds(request->arg("time").c_str()));
        response->printf("New time: %s", getTime());
        request->send(response);
     } else if (type == "Temp") {
        response->printf("%s %s",getTime(),getTemperature());
        request->send(response);
     }
    } else request->send(SPIFFS, "/index.html");
  });
  server.onNotFound([](AsyncWebServerRequest *request){request->send(404);});
  server.serveStatic("/", SPIFFS, "/");
  server.begin();
  Serial.println("HTTP server started");
  delay(100);
  xTaskCreatePinnedToCore (
      serialListener, /* Function to implement the task */
      "WiFi-WEBServer", /* Name of the task */
      10000,  /* Stack size in words */
      NULL,  /* Task input parameter */
      1,  /* Priority of the task */
      NULL,  /* Task handle. */
      0); /* Core where the task should run */
}
void loop() {
}
